// File: inputpane.h
/*     
   Translated to Spanish:
   Jorge Ruiz 
   COMP 499
   CSU Channel Islands

   SPANISH
   -------------------------------------------------------------------------------------
   PEP8-1 es una máquina virtual para la escritura de lenguaje de máquina y el conjunto   
            
   Copyright (C) 2009 J. Stanley Warford, Pepperdine University      
   
   Este programa es software libre: usted puede redistribuirlo y / o modificarlo      
   bajo los términos de la Licencia Pública General de GNU según es publicada por      
   la Free Software Foundation, bien de la versión 3 de la Licencia, o      
   (a su elección) cualquier versión posterior.      
   
   Este programa se distribuye con la esperanza de que sea útil,      
   pero SIN NINGUNA GARANTÍA, incluso sin la garantía implícita de      
   COMERCIALIZACIÓN o IDONEIDAD PARA UN PROPÓSITO PARTICULAR. Consulte la      
   GNU General Public License para más detalles.      
   
   Debería haber recibido una copia de la Licencia Pública General de GNU      
   junto con este programa. Si no es así, consulte <http://www.gnu.org/licenses/>.
*/
/*
    Pep8-1 is a virtual machine for writing machine language and assembly
    language programs.
    
    Copyright (C) 2009  J. Stanley Warford, Pepperdine University

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef INPUTPANE_H
#define INPUTPANE_H

#include <QtGui/QWidget>

namespace Ui {
    class InputPane;
}

class InputPane : public QWidget {
    Q_OBJECT
    Q_DISABLE_COPY(InputPane)
public:
    explicit InputPane(QWidget *parent = 0);
    virtual ~InputPane();

    QString toPlainText();
    // Post: input text edit text is returned

    void setText(QString input);
    // Post: the input pane text edit text is set to string

    void highlightOnFocus();
    // Post: Highlights the label based on the label window color saved in the UI file

    bool hasFocus();
    // Post: returns if the pane has focus

    void undo();
    // Post: the last action in the text edit is undone

    void redo();
    // Post: the last undo in the text edit is redone

    bool isUndoable();
    // Returns the undoability of the text edit

    bool isRedoable();
    // Returns the redoability of the text edit

    void cut();
    // Post: selected text in the text edit is cut to the clipboard

    void copy();
    // Post: selected text in the text edit is copied to the clipboard

    void paste();
    // Post: selected text in the clipboard is pasted to the text edit

    void setFont();
    // Post: the font used by the text edit is set to a font chosen in a font dialog

    void setReadOnly(bool b);
    // Post: the text edit's read only attribute is set to b

    void tab();
    // Post: a tab is inserted in the input pane if it is not read only

private:
    Ui::InputPane *ui;

    void mouseReleaseEvent(QMouseEvent *);

signals:
    void undoAvailable(bool);
    void redoAvailable(bool);

};

#endif // INPUTPANE_H
